@@ -373,7 +373,6 @@ static void listenonaddresses(const struct cvsd_addrs *addrs)
   int sock;
   char host[80],serv[40];
   int i;
-  struct addrinfo *last=NULL;
   struct addrinfo *addr;
   int count=0;
 
@@ -425,6 +424,18 @@ static void listenonaddresses(const struct cvsd_addrs *addrs)
       exit(1);
     }
 
+    /* force IPv6 sockets to not handle IPv4 traffic */
+#ifdef IPV6_V6ONLY
+    flag=1;
+    if ((addr->ai_family==AF_INET6)&&(setsockopt(sock,IPPROTO_IPV6,IPV6_V6ONLY,&flag,sizeof(int))!=0))
+    {
+      log_log(LOG_ERR,"setsockopt(IPV6_V6ONLY) failed: %s",strerror(errno));
+      if (close(sock))
+        log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
+      exit(1);
+    }
+#endif /* IPV6_V6ONLY */
+
 #ifdef TODO
     /* close the socket fast (SO_LINGER) */
     struct linger {
@@ -459,18 +470,9 @@ static void listenonaddresses(const struct cvsd_addrs *addrs)
     /* bind the socket to the specified port */
     if (bind(sock,addr->ai_addr,addr->ai_addrlen)<0)
     {
-      if ((errno==EADDRINUSE)&&(last!=NULL)&&
-          (last->ai_family==AF_INET6)&&
-          (addr->ai_family==AF_INET))
-      {
-        log_log(LOG_DEBUG,"debug: bind() failed (ignored): %s",strerror(errno));
-        if (close(sock))
-          log_log(LOG_WARNING,"problem closing socket (ignored): %s",strerror(errno));
-        continue;
-      }
       log_log(LOG_ERR,"bind() failed: %s",strerror(errno));
       if (close(sock))
-        log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
+        log_log(LOG_WARNING,"problem closing socket (ignored): %s",strerror(errno));
       exit(1);
     }
 
@@ -509,7 +511,6 @@ static void listenonaddresses(const struct cvsd_addrs *addrs)
         break;
     }
 
-    last=addr;
     count++;
   } /* next addr */
 
